@@ -24,7 +24,6 @@ import platform
 import socket
 import sys
 
-from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from oslo_utils import importutils
 
@@ -35,14 +34,13 @@ from os_brick import utils
 
 LOG = logging.getLogger(__name__)
 
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
-
 # List of connectors to call when getting
 # the connector properties for a host
 windows_connector_list = [
     'os_brick.initiator.windows.base.BaseWindowsConnector',
     'os_brick.initiator.windows.iscsi.WindowsISCSIConnector',
     'os_brick.initiator.windows.fibre_channel.WindowsFCConnector',
+    'os_brick.initiator.windows.rbd.WindowsRBDConnector',
     'os_brick.initiator.windows.smbfs.WindowsSMBFSConnector'
 ]
 
@@ -63,6 +61,7 @@ unix_connector_list = [
     'os_brick.initiator.connectors.vmware.VmdkConnector',
     'os_brick.initiator.connectors.storpool.StorPoolConnector',
     'os_brick.initiator.connectors.nvmeof.NVMeOFConnector',
+    'os_brick.initiator.connectors.lightos.LightOSConnector',
 ]
 
 
@@ -113,6 +112,8 @@ _connector_mapping_linux = {
         'os_brick.initiator.connectors.nvmeof.NVMeOFConnector',
     initiator.NVMEOF:
         'os_brick.initiator.connectors.nvmeof.NVMeOFConnector',
+    initiator.LIGHTOS:
+        'os_brick.initiator.connectors.lightos.LightOSConnector',
 }
 
 # Mapping for the S390X platform
@@ -159,6 +160,8 @@ _connector_mapping_windows = {
         'os_brick.initiator.windows.iscsi.WindowsISCSIConnector',
     initiator.FIBRE_CHANNEL:
         'os_brick.initiator.windows.fibre_channel.WindowsFCConnector',
+    initiator.RBD:
+        'os_brick.initiator.windows.rbd.WindowsRBDConnector',
     initiator.SMBFS:
         'os_brick.initiator.windows.smbfs.WindowsSMBFSConnector',
 }
@@ -207,8 +210,8 @@ def get_connector_properties(root_helper, my_ip, multipath, enforce_multipath,
     :param multipath: Enable multipath?
     :type multipath: bool
     :param enforce_multipath: Should we enforce that the multipath daemon is
-                              running?  If the daemon isn't running then the
-                              return dict will have multipath as False.
+                              running?  If the daemon isn't running then raise
+                              ProcessExecutionError to the caller.
     :type enforce_multipath: bool
     :param host: hostname.
     :param execute: execute helper.
