@@ -167,7 +167,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         host_paths = self._get_host_devices(possible_devs)
         return host_paths
 
-    def get_volume_paths(self, connection_properties: dict) -> list:
+    def get_volume_paths(self, connection_properties: dict) -> list[str]:
         volume_paths = []
         # first fetch all of the potential paths that might exist
         # how the FC fabric is zoned may alter the actual list
@@ -203,6 +203,10 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                         {'props': connection_properties})
             raise exception.VolumePathsNotFound()
 
+    def supports_multipath(self):
+        return self._linuxscsi.is_multipath_running(
+            root_helper=self._root_helper)
+
     @utils.trace
     @utils.connect_volume_prepare_result
     @base.synchronized('connect_volume', external=True)
@@ -218,6 +222,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         target_wwn - World Wide Name
         target_lun - LUN id of the volume
         """
+        self.check_multipath(connection_properties)
         device_info = {'type': 'block'}
 
         connection_properties = self._add_targets_to_connection_properties(
@@ -303,7 +308,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         device_info['path'] = device_path
         return device_info
 
-    def _get_host_devices(self, possible_devs: list) -> list:
+    def _get_host_devices(self, possible_devs: list) -> list[str]:
         """Compute the device paths on the system with an id, wwn, and lun
 
         :param possible_devs: list of (platform, pci_id, wwn, lun) tuples
@@ -320,7 +325,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         return host_devices
 
     def _get_possible_devices(self, hbas: list, targets: list,
-                              addressing_mode: Optional[str] = None) -> list:
+                              addressing_mode: Optional[str] = None) -> \
+            list[tuple[str, Any, str, int]]:
         """Compute the possible fibre channel device options.
 
         :param hbas: available hba devices.
